<?xml version="1.0" encoding="UTF-8"?>
<book version="5.0" xmlns="http://docbook.org/ns/docbook" xmlns:xi="http://www.w3.org/2001/XInclude">

  <#assign locale = "en" />
  <#assign sysDateTime = .now>

  <#assign left_header_text = ''/> 
	<#assign central_header_text = "Annotations report" />
	<#assign right_header_text = ''/> 
	
	<#assign left_footer_text = sysDateTime?string["dd/MM/yyyy"] + " - IUCLID 6 " + iuclid6Version!/> 
	<#assign central_footer_text = 'Annotations references report' />
	<#assign right_footer_text = ''/> 

<#import "utils.ftl" as utils>
<#-- Import common macros and functions -->
<#import "macros_common_general.ftl" as com>

<@com.initializeMainVariables />

<#assign rootEntity = entity.root>
<#-- Utility list to remember tarversed entities -->
<#assign traveresedEntityKeyList = [rootEntity.documentKey]>
<#-- Utility list to remember entities yet to be traversed -->
<#assign toTravereseEntityList = []>

<info>
      <title>Annotations report</title>
		<cover>
			<para role="cover.rule">
        <guiicon>

          <para role="i6header4">
            <#if _subject.documentType=="MIXTURE">Mixture: <@com.text _subject.MixtureName/>
            <#elseif _subject.documentType=="SUBSTANCE">Substance: <@com.text _subject.ChemicalName/>
            </#if>

            <#if _dossierHeader?has_content>
              <#assign dossier = _dossierHeader />

            <@com.emptyLine/>

            <para>
              <emphasis role="bold">Dossier details:</emphasis>
                <para role="indent">
                <itemizedlist>
                    <listitem>Dossier name: <@com.text dossier.name /></listitem>
                    <listitem>Dossier UUID: <@com.text dossier.subjectKey /></listitem>
                    <listitem>Dossier submission remarks: <@com.text dossier.remarks /></listitem>
                    <listitem>Dossier creation date and time: <@com.text dossier.creationDate /></listitem>
                    <listitem>Submission type: <@com.text dossier.submissionType /></listitem>
                    <#if ownerLegalEntity?has_content>
                      <listitem>Submitting legal entity:
                        <@com.text ownerLegalEntity.GeneralInfo.LegalEntityName/>
                      </listitem>
                    </#if>
                </itemizedlist>
                </para>
            </para>

            </#if>

          </para>

        </guiicon>
      </para>
    </cover>
 </info>
 
<chapter label="1">
		<title role="HEAD-1">List of annotations</title>

    <#-- Maxloop sets the limit at which the component hierarchy is travered -->
<#assign maxLoop=5>
  <table border="1">
    <title>Table of annotations</title>
      <col width="12.5%" />
      <col width="12.5%" />
      <col width="12.5%" />
      <col width="12.5%" />
      <col width="12.5%" />
      <col width="12.5%" />
      <col width="12.5%" />
      <col width="12.5%" />
      <tbody>
        <tr>								
          <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Section name and number</emphasis></th>
          <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Document name</emphasis></th>          
          <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Document uuid</emphasis></th>
          <th colspan="1"><?dbfo bgcolor="#F9C514" ?><emphasis role="bold">annotation type</emphasis></th>
          <th colspan="1"><?dbfo bgcolor="#F9C514" ?><emphasis role="bold">annotation name</emphasis></th>
          <th colspan="1"><?dbfo bgcolor="#F9C514" ?><emphasis role="bold">annotation status</emphasis></th>
          <th colspan="1"><?dbfo bgcolor="#F9C514" ?><emphasis role="bold">annotation agreement</emphasis></th>          
          <th colspan="1"><?dbfo bgcolor="#F9C514" ?><emphasis role="bold">annex of annotation</emphasis></th>       
        </tr>

        <#-- Traverse the main entity -->
        <@traverseEntity rootEntity />

          <#list 1..maxLoop as x>           
            <#-- Remove documents that have been already traversed -->
            <#assign toTravereseEntityList = utils.removeDocumentWitKey(toTravereseEntityList, traveresedEntityKeyList) >
            <#if !(toTravereseEntityList?has_content)>
              <#-- Stop looping if there is nothing to traverse -->
              <#break>
            </#if>
            <#-- Travers each entity from the list -->
            <#list toTravereseEntityList as entityItem>
              <#assign traveresedEntityKeyList= traveresedEntityKeyList + [entityItem.documentKey]>
                <@traverseEntity entityItem />
            </#list>          
          </#list>
        </tbody>
    </table>

<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->

<#macro traverseEntity entityDoc>
  <#if utils.isCompositeEntityDocument(entityDoc)>
    <@traverseCompositeEntity entityDoc />
  <#else>
    <@traverseSimpleEntity entityDoc />
  </#if>
</#macro>

<#macro traverseSimpleEntity entityDoc>
  <@traverseDoc entityDoc entityDoc 0 />
</#macro>

<#macro traverseCompositeEntity entityDoc>
  <#local tableOfContents=iuclid.localizeTreeFor(entityDoc.documentType, entityDoc.submissionType, entityDoc.documentKey, utils.getInheritedTemplates(entityDoc), utils.getRelatedCategories(entityDoc), utils.getRelatedMixtures(entityDoc))>
  <@traverseToc tableOfContents entityDoc 0 />
</#macro>

<#macro traverseToc sectionNode entityDocument level=0>
  <#if sectionNode?has_content>
    <#-- iterate through the section documents under this section if there are any -->
    <#if sectionNode.content?has_content>
      <#list sectionNode.content as doc>
        <#-- traverse through the document's blocks and fields aka elements -->
        <@traverseDoc doc entityDocument level+1 sectionNode />
      </#list>
    </#if>
    <#-- iterate through the child sections of the node and do a recursive call -->
    <#if sectionNode?children?has_content>
      <#list sectionNode?children as child>
        <@traverseToc child entityDocument level+1 />
      </#list>
    </#if>
  </#if>
</#macro>

<#macro traverseDoc docElementNode entityDocument level=0 sectionNode="" isDocument=true>
  <#if docElementNode?has_content>
    <#-- Annotations are only at document level -->
    <#if isDocument>
      <@printAnnotationsForDocument docElementNode entityDocument sectionNode />
    </#if>
    <#-- Check if the element node is a reference field (single or multiple) and remember referenced entities for later traversal in toTravereseEntityList -->
    <#assign toTravereseEntityList = utils.checkAndRememberReferencedEntity(docElementNode, toTravereseEntityList, traveresedEntityKeyList) />
    <#-- iterate through the child structure elements of this document element if there are any and do a recursive call -->
    <#if docElementNode?children?has_content>
      <#list docElementNode?children as child>
        <@traverseDoc child entityDocument level+1 sectionNode false/>
      </#list>
    </#if>
  </#if>
</#macro>

<#macro printAnnotationsForDocument sectionDoc entityDocument sectionNode>

  <#local sectionName = "">
  <#if sectionNode?has_content>
    <#if sectionNode.number?has_content>
      <#local sectionName = sectionNode.number + " ">
    </#if>
    <#local sectionName = sectionName + sectionNode.title>
  </#if>

  <#local sectionDocType = sectionDoc.documentType>

  <#if !utils.isEntityDocument(sectionDoc)>
    <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
  </#if>

  <#-- The web URL pointing to the document -->
  <#local docUrl=utils.getDocumentUrl(sectionDoc) />

  <#-- Get the annotations of the document and print a row for each of them -->
  <#local annotationKeys = sectionDoc.annotations />
  <#if annotationKeys?has_content>
    <#list annotationKeys as key>
      <#local annotation = iuclid.getDocumentForKey(key) />
      <#local annotationStatus = annotation.AdminInfo.AnnotationStatus />
      <#local annotationAgreement = annotation.EvalInfo.AgreementWithApplicantsSummary />

        <@tableOutput annotation annotationStatus annotationAgreement sectionName sectionDoc docUrl />

    </#list>
  </#if>
</#macro>

<#macro tableOutput annotation annotationStatus annotationAgreement sectionName sectionDoc docUrl>

 <tr>  
     <#-- IUCLID Section name and number -->
    <td>
      <@com.text sectionName />
    </td>
    
    <#-- IUCLID document name -->
    <td>
        <ulink url="${docUrl}">
          <@com.text sectionDoc.name />
        </ulink> 
    </td>

     <#-- IUCLID document uuid -->
    <td>
        <@com.text sectionDoc.documentKey.uuid />
    </td>

    <#-- annotation type -->
    <td>
        <@com.picklist annotation.AdminInfo.AnnotationType />
    </td>
    
    <#-- annotation name -->									
    <td>   
    <#if annotation.AdminInfo.Name?has_content>
      <@com.text annotation.AdminInfo.Name/>      
    </#if>
    
    <#assign historyList = (getFirstHistoryOnlyOfAnnotation(annotation))!"" />
    <#if historyList?has_content>
      <para><emphasis role="bold">Annotation Last updated:</emphasis> ${historyList.date}</para>
    </#if>    
    </td>

     <#-- annotation status  -->
     <td>
      <#if annotationStatus?has_content>
        <@com.picklist annotationStatus />
      </#if>
    </td>

    <#-- annotation agreement  -->
    <td>
      <#if annotationAgreement?has_content>
        <@com.picklist annotationAgreement/>
      </#if>
    </td>

    <#-- annotation annex  -->
    <td>
	  	<@utils.annexForAnnotations annotation />
    </td>

  </tr>
</#macro>

<#function getFirstHistoryOnlyOfAnnotation annotation>

<#local historyList = iuclid.getHistoryForDocumentKey(annotation.documentKey) />
	
	<#if !(annotation)?has_content>
		<#return [] />
	</#if>
	
	<#if !(historyList)?has_content>
		<#return [] />
	</#if>
	
	<#if historyList?has_content>		
		<#list historyList as history>	
			<#if !(history)?has_content>
				<#return [] />	
				<#else/>
					<#return historyList[0] />					
			</#if>
		</#list>
	</#if>		
</#function>

</chapter>

<chapter label="2">
		<title role="HEAD-1">Annex of Information on Annotations</title>			
			<para>
				<#include "annex_annotations.ftl" encoding="UTF-8" />
			</para>
</chapter>

</book>